@@ -16,8 +16,9 @@ use Test::MockObject;
 use Log::Log4perl qw(:easy);
 use Data::Dumper;
 use File::MMagic;
+use OAuth::Cmdline::GoogleDrive;
 
-our $VERSION = "0.08";
+our $VERSION = "0.09";
 
 ###########################################
 sub new {
@@ -25,18 +26,13 @@ sub new {
     my($class, %options) = @_;
 
     my $self = {
-        config_file => undef,
-        cfg         => undef,
+        init_done       => undef,
         api_file_url    => "https://www.googleapis.com/drive/v2/files",
         api_upload_url  => "https://www.googleapis.com/upload/drive/v2/files",
+        oauth           => OAuth::Cmdline::GoogleDrive->new( ),
         %options,
     };
 
-    if( ! $self->{ config_file } ) {
-        my( $home )  = glob "~";
-        $self->{ config_file } = "$home/.google-drive.yml";
-    }
-
     bless $self, $class;
 }
 
@@ -45,67 +41,20 @@ sub init {
 ###########################################
     my( $self, $path ) = @_;
 
-    if( $self->{ init_done } and
-        ! $self->token_expired() ) {
+    if( $self->{ init_done } ) {
         return 1;
     }
 
-    my $cfg = {};
-
-    if( ! -f $self->{ config_file } ) {
-        LOGDIE "$self->{ config_file } not found.";
-    }
-
-    $cfg = LoadFile $self->{ config_file };
-    $self->{ cfg } = $cfg;
-
-    $self->token_refresh( $cfg );
-
-    DEBUG "Testing API with refreshed token";
+    DEBUG "Testing API";
     if( !$self->api_test() ) {
-        LOGDIE "api_test failed after token refresh";
+        LOGDIE "api_test failed";
     }
 
-    DumpFile( $self->{ config_file }, $cfg );
-    $self->{ cfg } = $cfg;
-
     $self->{ init_done } = 1;
 
     return 1;
 }
 
-###########################################
-sub token_expire {
-###########################################
-    my( $self ) = @_;
-
-      # expire the token
-    $self->{ cfg }->{ expires } = time() - 1;
-}
-
-###########################################
-sub token_expired {
-###########################################
-    my( $self ) = @_;
-
-    my $time_remaining = $self->{ cfg }->{ expires } - time();
-
-    if( $time_remaining < 300 ) {
-
-        if( $time_remaining < 0 ) {
-            INFO "Token expired ", -$time_remaining, " seconds ago";
-        } else {
-            INFO "Token will expire in $time_remaining seconds";
-        }
-
-        INFO "Token needs to be refreshed.";
-
-        return 1;
-    }
-
-    return 0;
-}
-
 ###########################################
 sub api_test {
 ###########################################
@@ -117,10 +66,8 @@ sub api_test {
 
     my $req = HTTP::Request->new(
         GET => $url->as_string,
-        HTTP::Headers->new( Authorization =>
-            "Bearer " . $self->{ cfg }->{ access_token })
     );
-
+    $req->header( $self->{ oauth }->authorization_headers() );
     DEBUG "Fetching $url";
 
     my $resp = $ua->request( $req );
@@ -180,14 +127,9 @@ sub files {
     while( 1 ) {
         my $url = $self->file_url( $opts );
         my $data = $self->http_json( $url );
+        my $next_item = $self->item_iterator( $data );
 
-        for my $item ( @{ $data->{ items } } ) {
-
-            # ignore trash
-          if( $item->{ labels }->{ trashed } ) {
-              DEBUG "Skipping $item->{ title } (trashed)";
-          }
-
+        while( my $item = $next_item->() ) {
           if( $item->{ kind } eq "drive#file" ) {
             my $file = $item->{ originalFilename };
             if( !defined $file ) {
@@ -264,7 +206,7 @@ sub file_upload {
 
     my $req = &HTTP::Request::Common::PUT(
         $url->as_string,
-        Authorization  => "Bearer " . $self->{ cfg }->{ access_token },
+        $self->{ oauth }->authorization_headers(),
         "Content-Type" => $mime_type,
         Content        => $file_data,
     );
@@ -311,8 +253,9 @@ sub children_by_folder_id {
         $url->query_form( $opts );
 
         my $data = $self->http_json( $url );
+        my $next_item = $self->item_iterator( $data );
 
-        for my $item ( @{ $data->{ items } } ) {
+        while( my $item = $next_item->() ) {
             push @children, $self->data_factory( $item );
         }
 
@@ -406,8 +349,9 @@ sub search {
         $url->query_form( $opts );
 
         my $data = $self->http_json( $url );
+        my $next_item = $self->item_iterator( $data );
 
-        for my $item ( @{ $data->{ items } } ) {
+        while( my $item = $next_item->() ) {
             push @children, $self->data_factory( $item );
         }
 
@@ -447,9 +391,8 @@ sub download {
 
     my $req = HTTP::Request->new(
         GET => $url,
-        HTTP::Headers->new( Authorization =>
-            "Bearer " . $self->{ cfg }->{ access_token })
-      );
+    );
+    $req->header( $self->{ oauth }->authorization_headers() );
 
     my $ua = LWP::UserAgent->new();
     my $resp = $ua->request( $req, $local_file );
@@ -467,43 +410,6 @@ sub download {
 }
 
 
-###########################################
-sub token_refresh {
-###########################################
-  my( $self, $cfg ) = @_;
-
-  DEBUG "Refreshing access token";
-
-  my $req = &HTTP::Request::Common::POST(
-    'https://accounts.google.com/o' .
-    '/oauth2/token',
-    [
-      refresh_token =>
-        $cfg->{ refresh_token },
-      client_id     =>
-        $cfg->{ client_id },
-      client_secret =>
-        $cfg->{ client_secret },
-      grant_type    => 'refresh_token',
-    ]
-  );
-
-  my $resp = $self->http_loop( $req, 1 );
-
-  if ( $resp->is_success() ) {
-    my $data = from_json( $resp->content() );
-    $cfg->{ access_token } =
-      $data->{ access_token };
-    $cfg->{ expires } =
-      time() + $data->{ expires_in };
-    DEBUG "Token refreshed, will expire in $data->{ expires_in } seconds";
-    return 1;
-  }
-
-  ERROR "Token refresh failed: ", $resp->status_line();
-  return undef;
-}
-
 ###########################################
 sub http_loop {
 ###########################################
@@ -552,16 +458,15 @@ sub http_json {
     if( $post_data ) {
         $req = &HTTP::Request::Common::POST(
             $url->as_string,
-            Authorization => "Bearer " . $self->{ cfg }->{ access_token },
+            $self->{ oauth }->authorization_headers(),
             "Content-Type"=> "application/json",
             Content       => to_json( $post_data ),
         );
     } else {
       $req = HTTP::Request->new(
         GET => $url->as_string,
-        HTTP::Headers->new( Authorization =>
-            "Bearer " . $self->{ cfg }->{ access_token })
       );
+      $req->header( $self->{ oauth }->authorization_headers() );
     }
 
     my $resp = $self->http_loop( $req );
@@ -586,6 +491,33 @@ sub file_mime_type {
     return $self->{ magic }->checktype_filename( $file );
 }
 
+###########################################
+sub item_iterator {
+###########################################
+    my( $self, $data ) = @_;
+
+    my $idx = 0;
+
+    if( !defined $data ) {
+        die "no data in item_iterator";
+    }
+
+    return sub {
+        {
+            my $next_item = $data->{ items }->[ $idx++ ];
+
+            return if !defined $next_item;
+
+            if( $next_item->{ labels }->{ trashed } ) {
+                DEBUG "Skipping $next_item->{ title } (trashed)";
+                redo;
+            }
+
+            return $next_item;
+        }
+    };
+}
+
 1;
 
 __END__
@@ -632,7 +564,7 @@ distribution.
 Before you run it, you need to register your 'app' with Google Drive
 and obtain a client_id and a client_secret from Google:
 
-    https://developers.google.com/drive
+    https://developers.google.com/drive/web/enable-sdk
 
 Click on "Enable the Drive API and SDK", and find "Create an API project in
 the Google APIs Console". On the API console, create a new project, click
